﻿/*
 * 作者：Leslie Xin
 * 邮箱：lesliexin@outlook.com
 * 网站：www.lesliexin.com
 * －－
 * 版权所有
 * 使用代码需附上作者信息
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Runtime.InteropServices;

namespace DemoControls
{
    /// <summary>
    /// LProgress
    /// </summary>
    public partial class LProgress : Form
    {
        /// <summary>
        /// LProgress
        /// </summary>
        public LProgress()
        {
            InitializeComponent();
        }

        /// <summary>
        /// LProgress
        /// </summary>
        /// <param name="l_Title">标题</param>
        /// <param name="l_DefaultMainText">默认显示的主文本</param>
        /// <param name="l_CancelVisible">是否显示取消按钮，默认：显示</param>
        /// <param name="l_MainPBarMode">进度条模式</param>
        /// <param name="l_Argument">自定义参数</param>
        public LProgress(string l_Title, string l_DefaultMainText, bool l_CancelVisible, ProgressMode l_MainPBarMode, object l_Argument)
        {
            InitializeComponent();
            L_Title = l_Title;
            L_DefaultMainText = l_DefaultMainText;
            L_CancelVisible = l_CancelVisible;
            L_MainPBarMode = l_MainPBarMode;
            L_Argument = l_Argument;
        }


        #region PInvoke
        [DllImport("user32.dll")]
        private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);
        [DllImport("user32.dll")]
        static extern bool RemoveMenu(IntPtr hMenu, uint uPosition, uint uFlags);
        private const uint SC_CLOSE = 0xF060;
        private const int MF_REMOVE = 0x1000;
        /// <summary>
        /// 使窗体关闭按钮变灰不可用
        /// </summary>
        /// <param name="hWnd">窗体句柄</param>
        public static void CloseButtonDisabled(IntPtr hWnd)
        {
            IntPtr hMenu = GetSystemMenu(hWnd, false);
            if (hMenu != IntPtr.Zero)
            {
                RemoveMenu(hMenu, SC_CLOSE, MF_REMOVE);
            }
        }
        #endregion

        #region Event

        /// <summary>
        /// 委托
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void L_DoEventHandler(LProgress sender, LEventArgs e);
        /// <summary>
        /// 事件
        /// </summary>
        public event L_DoEventHandler L_Do;

        #endregion

        #region Public

        /// <summary>
        /// 主进度条模式<para>默认：Known（Continuous）</para>
        /// </summary>
        public ProgressMode L_MainPBarMode
        {
            get { return lPBar_Main.L_ProgressMode; }
            set { lPBar_Main.L_ProgressMode = value; }
        }

        /// <summary>
        /// 是否显示取消按钮<para>默认：显示</para>
        /// </summary>
        public bool L_CancelVisible
        {
            set
            {
                btn_Cancel.Visible = value;
                this.Size = new Size(this.Width, value ? 155 : 115);
            }
        }

        /// <summary>
        /// 是否显示子进度条<para>默认：不显示</para>
        /// </summary>
        public bool L_ChildPBarVisiable
        {
            set
            {
                lPBar_Child.Visible = value;
            }
        }

        /// <summary>
        /// 自定义参数
        /// </summary>
        public object L_Argument { get; set; }

        /// <summary>
        /// 显示标题
        /// </summary>
        public string L_Title
        {
            get
            {
                return Text;
            }
            set
            {
                this.Text = value;
            }
        }

        /// <summary>
        /// 默认显示主文本
        /// </summary>
        public string L_DefaultMainText { get; set; }
        
        #endregion

        #region Private
        private int iLastMainPercent;
        private int iLastChildPercent;
        private string sLastMainMsg;
        private string sLastChildMsg;
        private Thread tWork;
        #endregion

        #region Func_Public

        /// <summary>
        /// 仅更新主文本
        /// </summary>
        /// <param name="sMsg">主文本</param>
        public void L_pSetMainProgress(string sMsg)
        {
            if (sMsg != sLastMainMsg)
            {
                sLastMainMsg = sMsg;
                pChangeMainString(sMsg);
            }
        }

        /// <summary>
        /// 仅更新子文本
        /// </summary>
        /// <param name="sMsg">子文本</param>
        public void L_pSetChildProgress(string sMsg)
        {
            if (sMsg != sLastChildMsg)
            {
                sLastChildMsg = sMsg;
                pChangeChildString(sMsg);
            }
        }

        /// <summary>
        /// 仅更新主进度
        /// </summary>
        /// <param name="iPercent">范围：0-100</param>
        public void L_pSetMainProgress(int iPercent)
        {
            if(iPercent!=iLastMainPercent)
            {
                iLastMainPercent = iPercent;
                pChangeMainPBarValue(iPercent);
            }
        }

        /// <summary>
        /// 更新子进度条进度
        /// </summary>
        /// <param name="iPercent">范围：0-100</param>
        public void L_pSetChildProgress(int iPercent)
        {
            if (iPercent != iLastChildPercent)
            {
                iLastChildPercent = iPercent;
                pChangeChildPBarValue(iPercent);
            }
        }

        /// <summary>
        /// 改变子进度条可见状态
        /// </summary>
        /// <param name="IsVisible">是否可见</param>
        public void L_pChildPBarVisible(bool IsVisible)
        {
            pChangeChildPBarVisible(IsVisible);
        }

        /// <summary>
        /// 改变子进度条样式
        /// </summary>
        /// <param name="IsKnown">Known/Unknown</param>
        public void L_pChildPBarMode(bool IsKnown)
        {
            pChangeChildPBarMode(IsKnown);
        }

        /// <summary>
        /// 退出
        /// </summary>
        /// <param name="bOKCancel">OK/Cancel</param>
        public void L_pExit(bool bOKCancel)
        {
            if (this.InvokeRequired)
            {
                if (this.IsDisposed || this.Disposing) return;
                this.Invoke(new Action<bool>(L_pExit), new object[] { bOKCancel });
            }
            else
            {
                if (tWork != null)
                    tWork.Abort();
                DialogResult = bOKCancel ? DialogResult.OK : DialogResult.Cancel;
            }
        }

        #endregion

        #region Func_Private
        private void pChangeMainString(string s)
        {
            if (lbl_Main.InvokeRequired)
            {
                if (lbl_Main.IsDisposed || lbl_Main.Disposing)
                    return;
                lbl_Main.Invoke(new Action<string>(pChangeMainString), new object[] { s });
            }
            else
            {
                lbl_Main.Text = s;
            }
        }
        private void pChangeChildString(string s)
        {
            if (lbl_Child.InvokeRequired)
            {
                if (lbl_Child.IsDisposed || lbl_Child.Disposing)
                    return;
                lbl_Child.Invoke(new Action<string>(pChangeChildString), new object[] { s });
            }
            else
            {
                lbl_Child.Text = s;
            }
        }
        private void pChangeMainPBarValue(int i)
        {
            if (lPBar_Main.InvokeRequired)
            {
                if (lPBar_Main.IsDisposed || lPBar_Main.Disposing)
                    return;
                lPBar_Main.Invoke(new Action<int>(pChangeMainPBarValue), new object[] { i });
            }
            else
            {
                lPBar_Main.L_Value = i;
            }
        }
        private void pChangeChildPBarValue(int i)
        {
            if (lPBar_Child.InvokeRequired)
            {
                if (lPBar_Child.IsDisposed || lPBar_Child.Disposing)
                    return;
                lPBar_Child.Invoke(new Action<int>(pChangeChildPBarValue), new object[] { i });
            }
            else
            {
                lPBar_Child.L_Value = i;
            }
        }
        private void pChangeChildPBarVisible(bool IsVisible)
        {
            if (lPBar_Child.InvokeRequired)
            {
                if (lPBar_Child.IsDisposed || lPBar_Child.Disposing)
                    return;
                lPBar_Child.Invoke(new Action<bool>(pChangeChildPBarVisible), new object[] { IsVisible });
            }
            else
            {
                lPBar_Child.Visible = IsVisible;
            }
        }
        private void pChangeChildPBarMode(bool IsKnown)
        {
            if (lPBar_Child.InvokeRequired)
            {
                if (lPBar_Child.IsDisposed || lPBar_Child.Disposing)
                    return;
                lPBar_Child.Invoke(new Action<bool>(pChangeChildPBarMode), new object[] { IsKnown });
            }
            else
            {
                pChildPBarWave(!IsKnown);
                if (IsKnown)
                {
                    lPBar_Child.L_ProgressMode = ProgressMode.Known;
                    lPBar_Child.L_ProgressStyle = ProgressStyle.Continuous;
                }
                else
                {
                    lPBar_Child.L_ProgressMode = ProgressMode.Unknown;
                    lPBar_Child.L_ProgressStyle = ProgressStyle.Helical;
                }
            }
        }
        private void ToDoWork()
        {
            L_Do?.Invoke(this, new LEventArgs(L_Argument));
            L_pExit(true);
        }
        private void pMainPBarWave(bool start)
        {
            if (lPBar_Main.InvokeRequired)
            {
                if (lPBar_Main.IsDisposed || lPBar_Main.Disposing)
                    return;
                lPBar_Main.Invoke(new Action<bool>(pMainPBarWave), new object[] { start });
            }
            else
            {
                if (start) lPBar_Main.L_StartWave();
                else lPBar_Main.L_StopWave();
            }
        }
        private void pChildPBarWave(bool start)
        {
            if (lPBar_Child.InvokeRequired)
            {
                if (lPBar_Child.IsDisposed || lPBar_Child.Disposing)
                    return;
                lPBar_Child.Invoke(new Action<bool>(pChildPBarWave), new object[] { start });
            }
            else
            {
                if (start) lPBar_Child.L_StartWave();
                else lPBar_Child.L_StopWave();
            }
        }
        #endregion

        private void btn_Cancel_Click(object sender, EventArgs e)
        {
            btn_Cancel.Enabled = false;
            lPBar_Main.L_StopWave();
            L_pExit(false);
        }

        private void LProgress_Load(object sender, EventArgs e)
        {
            CloseButtonDisabled(this.Handle);

            lbl_Main.Text = L_DefaultMainText;
            iLastMainPercent = 0;
            sLastMainMsg = L_DefaultMainText;

            pMainPBarWave(true);
            tWork = new Thread(ToDoWork);
            tWork.Start();
        }
    }
}
