﻿/*
 * 作者：Leslie Xin
 * 邮箱：lesliexin@outlook.com
 * 网站：www.lesliexin.com
 * －－
 * 版权所有
 * 使用代码需附上作者信息
 * 
 */

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace DemoControls
{
    /// <summary>
    /// 进度条
    /// </summary>
    public class LProgressBar:Control
    {
        /// <summary>
        /// 进度条
        /// </summary>
        public LProgressBar()
        {
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            SetStyle(ControlStyles.ResizeRedraw, true);
            SetStyle(ControlStyles.SupportsTransparentBackColor, true);

            BackColor = Color.Transparent;
            ForeColor = Color.Transparent;

            tTimer = new System.Windows.Forms.Timer();
            tTimer.Interval = 10;
            tTimer.Tick += TTimer_Tick;

            CreateControl();
        }

        #region Property

        private int _Value = 0;
        /// <summary>
        /// 当前进度值<para>范围：0~100</para>
        /// </summary>
        [Category("DemoUI"), Description("当前进度值\r\n范围：0~100")]
        public int L_Value
        {
            get { return _Value; }
            set
            {
                _Value = value;
                if (_Value < 0) _Value = 0;
                if (_Value > 100) _Value = 100;

                Invalidate();
            }
        }

        #region 绘制相关

        private int _SurroundLineSize = 0;
        /// <summary>
        /// 环绕线宽度<para>默认：0</para>
        /// </summary>
        [Category("DemoUI"), Description("环绕线宽度宽度\r\n默认：0")]
        public int L_SurroundLineSize
        {
            get { return _SurroundLineSize; }
            set
            {
                _SurroundLineSize = value;
                if (_SurroundLineSize < 0) _SurroundLineSize = 0;
                Invalidate();
            }
        }

        private Color _SurroundLineColor = Color.Empty;
        /// <summary>
        /// 环绕线颜色<para>默认：无颜色</para>
        /// </summary>
        [Category("DemoUI"), Description("环绕线颜色\r\n默认：无颜色")]
        public Color L_SurroundLineColor
        {
            get { return _SurroundLineColor; }
            set
            {
                _SurroundLineColor = value;
                Invalidate();
            }
        }

        private int _ShrinkSize = 0;
        /// <summary>
        /// 收缩宽度<para>默认：0</para>
        /// </summary>
        [Category("DemoUI"), Description("收缩宽度\r\n默认：0")]
        public int L_ShrinkSize
        {
            get { return _ShrinkSize; }
            set
            {
                _ShrinkSize = value;
                if (_ShrinkSize < 0) _ShrinkSize = 0;
                Invalidate();
            }
        }

        private Color _ShrinkColor = Color.FromArgb(128, 255, 128);
        /// <summary>
        /// 收缩后形状颜色<para>默认：浅绿色</para>
        /// </summary>
        [Category("DemoUI"), Description("收缩后形状颜色\r\n默认：浅绿色")]
        public Color L_ShrinkColor
        {
            get { return _ShrinkColor; }
            set
            {
                _ShrinkColor = value;
                Invalidate();
            }
        }

        private int _DrawMargin = 0;
        /// <summary>
        /// 进度条绘制边界与边界的距离<para>默认：0</para>
        /// </summary>
        [Category("DemoUI"), Description("进度条绘制边界与边界的距离\r\n默认：0")]
        public int L_DrawMargin
        {
            get { return _DrawMargin; }
            set
            {
                _DrawMargin = value;
                if (_DrawMargin < 0) _DrawMargin = 0;
                Invalidate();
            }
        }

        private int _ArcSize = 4;
        /// <summary>
        /// 圆环（Arc）样式下厚度<para>默认：4</para>
        /// </summary>
        [Category("DemoUI"), Description("Arc（圆环）样式下厚度\r\n默认：4")]
        public int L_ArcSize
        {
            get { return _ArcSize; }
            set
            {
                _ArcSize = value;
                if (_ArcSize < 0) _ArcSize = 0;
                Invalidate();
            }
        }

        private Color _SliderColor = Color.FromArgb(125, 0, 0, 0);
        /// <summary>
        /// 滑条颜色<para>默认：黑色半透明</para>
        /// </summary>
        [Category("DemoUI"), Description("滑条颜色\r\n默认：黑色半透明")]
        public Color L_SliderColor
        {
            get { return _SliderColor; }
            set
            {
                _SliderColor = value;
                Invalidate();
            }
        }
        
        private int _BlockSize = 15;
        /// <summary>
        /// 分块的宽度/度数，比例：有颜色/无颜色=2/1<para>默认：15，即有颜色10，无颜色5</para>
        /// </summary>
        [Category("DemoUI"), Description("像素/度数，其中比例：有颜色/无颜色=2/1\r\n默认：15，即有颜色10，无颜色5")]
        public int L_BlockSize
        {
            get { return _BlockSize; }
            set
            {
                _BlockSize = value;
                Invalidate();
            }
        }
        
        #endregion

        #region 样式相关

        private ProgressStyle _ProgressStyle = ProgressStyle.Continuous;
        /// <summary>
        /// 进度条样式<para>默认：Continuous（连续）</para>
        /// </summary>
        [Category("DemoUI"), Description("进度条样式\r\n默认：Continuous（连续）")]
        public ProgressStyle L_ProgressStyle
        {
            get { return _ProgressStyle; }
            set
            {
                _ProgressStyle = value;
                pChangeStyle();
                Invalidate();
            }
        }

        private ProgressType _ProgressType = ProgressType.Bar;
        /// <summary>
        /// 进度条类型<para>默认：Bar（条状）</para>
        /// </summary>
        [Category("DemoUI"), Description("进度条类型\r\n默认：Bar（条状）")]
        public ProgressType L_ProgressType
        {
            get { return _ProgressType; }
            set
            {
                _ProgressType = value;
                pChangeStyle();
                Invalidate();
            }
        }

        private ProgressMode _ProgressMode = ProgressMode.Known;
        /// <summary>
        /// 进度条模式<para>默认：Known（进度已知）</para>
        /// </summary>
        [Category("DemoUI"), Description("进度条模式\r\n默认：Known（进度已知）")]
        public ProgressMode L_ProgressMode
        {
            get { return _ProgressMode; }
            set
            {
                _ProgressMode = value;
                pChangeStyle();
                Invalidate();
            }
        }

        /// <summary>
        /// Marquee样式时从左到右花费时间，单位：ms<para>默认：1000ms（1秒）</para>
        /// </summary>
        [Category("DemoUI"), Description("Marquee样式时从左到右花费时间，单位：ms\r\n默认：1000ms（1秒）")]
        public int L_MarqueeTime { get; set; } = 1000;
        
        /// <summary>
        /// Marquee时滑块摆动样式<para>默认：Swing（左右摆动）</para>
        /// </summary>
        [Category("DemoUI"), Description("Marquee时滑块摆动样式\r\n默认：Swing（左右摆动）")]
        public MarqueeStyle L_MarqueeStyle { get; set; } = MarqueeStyle.Swing;
        
        /// <summary>
        /// Marquee时滑块运动类型<para>默认：SlowInSlowOut（缓入缓出）</para>
        /// </summary>
        [Category("DemoUI"), Description("Marquee时滑块运动类型\r\n默认：SlowInSlowOut（缓入缓出）")]
        public MarqueeType L_MarqueeType { get; set; } = MarqueeType.SlowInSlowOut;

        #endregion

        private ProgressText _ProgressText = ProgressText.None;
        /// <summary>
        /// 进度文本<para>默认：无</para>
        /// </summary>
        [Category("DemoUI"), Description("进度文本\r\n默认：无")]
        public ProgressText L_ProgressText
        {
            get { return _ProgressText; }
            set
            {
                _ProgressText = value;
                pChangeStyle();
                Invalidate();
            }
        }

        private Color _TextColor = Color.FromArgb(125, 0, 0, 0);
        /// <summary>
        /// 文本颜色<para>默认：黑色半透明</para>
        /// </summary>
        [Category("DemoUI"), Description("文本颜色\r\n默认：黑色半透明")]
        public Color L_TextColor
        {
            get { return _TextColor; }
            set
            {
                _TextColor = value;
                Invalidate();
            }
        }
        
        #endregion

        #region Private

        private double dMarqueeValue = 0;
        private bool bAcrossed = false;
        private bool bToRight = true;
        private System.Windows.Forms.Timer tTimer;

        #endregion

        #region Funcation

        /// <summary>
        /// 未知类型时开始摆动
        /// </summary>
        public void L_StartWave()
        {
            if (_ProgressMode == ProgressMode.Unknown)
            {
                tTimer.Start();
            }
        }

        /// <summary>
        ///  未知类型时停止摆动
        /// </summary>
        public void L_StopWave()
        {
            if (_ProgressMode == ProgressMode.Unknown)
            {
                tTimer.Stop();
            }
        }

        /// <summary>
        /// 改变进度条样式
        /// </summary>
        private void pChangeStyle()
        {
            if (_ProgressMode == ProgressMode.Unknown)
            {
                _ProgressText = ProgressText.None;
                if (_ProgressType == ProgressType.Bar)
                {
                    if (_ProgressStyle != ProgressStyle.Marquee && _ProgressStyle != ProgressStyle.Helical)
                    {
                        _ProgressStyle = ProgressStyle.Marquee;
                    }
                }
                else
                {
                    if (_ProgressStyle != ProgressStyle.Marquee)
                    {
                        _ProgressStyle = ProgressStyle.Marquee;
                    }
                }
            }
            else
            {
                if (_ProgressType == ProgressType.Bar)
                {
                    if (_ProgressStyle == ProgressStyle.Marquee)
                    {
                        _ProgressStyle = ProgressStyle.Continuous;
                    }
                }
                else
                {
                    if (_ProgressStyle == ProgressStyle.Marquee||_ProgressStyle==ProgressStyle.Helical)
                    {
                        _ProgressStyle = ProgressStyle.Continuous;
                    }
                }
            }
        }

        #endregion

        
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.SmoothingMode = SmoothingMode.HighQuality;

            //进度实际绘制宽度、高度
            float fRealWidth = Width - _DrawMargin * 2;
            float fRealHeight = Height - _DrawMargin * 2;

            float fRealLeftTop = _DrawMargin - 0.5f;
            if (fRealLeftTop < 0) fRealLeftTop = 0;

            //上下左右各留0.5个像素，防止切边
            bool bDrawSurroundLine = _SurroundLineSize > 0 ? true : false;
            bool bDrawInner = _ShrinkSize * 2 < Math.Min(Width, Height) ? true : false;
            float fSurroundLineSizeF = Convert.ToSingle(_SurroundLineSize);
            RectangleF rectInner = new RectangleF(_ShrinkSize, _ShrinkSize, (Width - 1) - _ShrinkSize * 2, (Height - 1) - _ShrinkSize * 2);
            RectangleF retcSurrondLine = new RectangleF(fSurroundLineSizeF / 2, fSurroundLineSizeF / 2, (Width - 1) - fSurroundLineSizeF, (Height - 1) - fSurroundLineSizeF);
            //弧线宽度为总宽度的十分之一，因为是圆弧，所以最终看起来是五分之一
            float fSurroundArcSize = _ArcSize > 0 ? _ArcSize : Width / 10;
            RectangleF retcSurrondArc = new RectangleF(fSurroundArcSize / 2 + fRealLeftTop, fSurroundArcSize / 2 + fRealLeftTop, (fRealWidth - 1) - fSurroundArcSize, (fRealHeight - 1) - fSurroundArcSize);

            SolidBrush sbInner = new SolidBrush(_ShrinkColor);
            SolidBrush sbFore = new SolidBrush(L_SliderColor);
            SolidBrush sbText = new SolidBrush(_TextColor);
            Pen penSurrondLine = new Pen(_SurroundLineColor, fSurroundLineSizeF);
            Pen penSurroundArc = new Pen(L_SliderColor, fSurroundArcSize);

            //滑块宽度为总宽度的五分之一
            float fSlider = Convert.ToSingle(fRealWidth) / 5;
            if (_ProgressType != ProgressType.Bar)
            {
                //360/5
                fSlider = 72;
            }

            float fRange = 0;
            if (L_MarqueeStyle == MarqueeStyle.Swing) fRange = fRealWidth - fSlider;
            else if (L_MarqueeStyle == MarqueeStyle.Cross) fRange = fRealWidth + fSlider;
            else fRange = fRealWidth;

            float fBlockSize = _BlockSize;
            if (fBlockSize < 3) fBlockSize = 3.0f;
            if (_ProgressType == ProgressType.Bar)
            {
                if (fBlockSize > fRealWidth) fBlockSize = fRealWidth;
            }
            else
            {
                if (fBlockSize > 360.0f) fBlockSize = 360.0f;
            }

            #region Bar
            if (_ProgressType == ProgressType.Bar)
            {
                //画内部
                if (bDrawInner)
                {
                    e.Graphics.FillRectangle(sbInner, rectInner);
                }

                //画线
                if (bDrawSurroundLine)
                {
                    e.Graphics.DrawRectangle(penSurrondLine, retcSurrondLine.X, retcSurrondLine.Y, retcSurrondLine.Width, retcSurrondLine.Height);
                }

                if (_ProgressMode == ProgressMode.Known)
                {

                    #region Continuous
                    if (_ProgressStyle == ProgressStyle.Continuous)
                    {
                        //画进度
                        float fProgress = Convert.ToSingle(_Value) * fRealWidth / 100;
                        e.Graphics.FillRectangle(sbFore, _DrawMargin - 0.5f, fRealLeftTop, fProgress, fRealHeight);
                    }
                    #endregion
                    #region Blocks
                    else if (_ProgressStyle == ProgressStyle.Blocks)
                    {
                        //获取方块数
                        float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                        int iShowBlocks = Convert.ToInt32(fBlocks * _Value / 100);
                        //画进度
                        for (int i = 0; i < iShowBlocks; i++)
                        {
                            e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                        }
                    }
                    #endregion
                    #region Helical
                    else if (_ProgressStyle == ProgressStyle.Helical)
                    {
                        float fProgress = Convert.ToSingle(_Value) * fRealWidth / 100;

                        //这里为了获得更好看的效果，要多加1个Block
                        float fBmpWidth = fRealWidth + fBlockSize;
                        float fBlocks = fBmpWidth / fBlockSize;

                        Bitmap bmp = new Bitmap(Convert.ToInt32(fBmpWidth), Convert.ToInt32(fRealHeight));
                        GraphicsPath gp = new GraphicsPath();
                        for (int i = 0; i < fBlocks; i++)
                        {
                            gp.AddPolygon(new PointF[]
                            {
                            //top-right
                            new PointF(fBmpWidth - i * fBlockSize ,0),
                            //bottom-right
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize,fRealHeight),
                            //bottom-left
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize*5/3,fRealHeight),
                            //top-left
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize*2/3,0),

                            });
                            gp.CloseFigure();
                        }
                        gp.CloseAllFigures();

                        Graphics g = Graphics.FromImage(bmp);
                        g.SmoothingMode = SmoothingMode.HighQuality;
                        g.FillPath(sbFore, gp);
                        gp.Dispose();
                        g.Dispose();
                        e.Graphics.DrawImage(bmp, new RectangleF(fRealLeftTop, fRealLeftTop, fProgress, fRealHeight), new RectangleF(bmp.Width - fProgress - fBlockSize, 0, fProgress, bmp.Height), GraphicsUnit.Pixel);
                        bmp.Dispose();
                    }
                    #endregion
                    #region BlocksToContinuous
                    else if (_ProgressStyle == ProgressStyle.BlocksToContinuous)
                    {
                        float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                        if (_Value <= 50)
                        {
                            int iShowBlocks = Convert.ToInt32(fBlocks * _Value * 2 / 100);
                            //画进度
                            for (int i = 0; i < iShowBlocks; i++)
                            {
                                e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                            }
                        }
                        else
                        {
                            int iShowBlocks = Convert.ToInt32(fBlocks * ((_Value - 50f) * 2 / 100));
                            for (int i = iShowBlocks; i < Convert.ToInt32(fBlocks); i++)
                            {
                                e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                            }

                            //画进度
                            float fProgress = Convert.ToSingle((_Value - 50) * 2) * fRealWidth / 100;
                            e.Graphics.FillRectangle(sbFore, fRealLeftTop, fRealLeftTop, fProgress, fRealHeight);

                        }
                    }
                    #endregion
                }
                else
                {
                    #region Marquee
                    if (_ProgressStyle == ProgressStyle.Marquee)
                    {
                        if (L_MarqueeType == MarqueeType.SlowInSlowOut)
                        {
                            if (L_MarqueeStyle == MarqueeStyle.Swing)
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                                e.Graphics.FillRectangle(sbFore, Convert.ToSingle(fz) + fRealLeftTop, fRealLeftTop, fSlider, fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Cross)
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2 - fSlider;
                                float fz2 = Convert.ToSingle(fz) + fRealLeftTop;
                                float fW = fSlider;
                                if (fz2 < fRealLeftTop)
                                {
                                    fW = fz2 + fSlider - (fRealLeftTop);
                                    fz2 = fRealLeftTop;
                                }
                                if (fz2 + fSlider > Width - (fRealLeftTop))
                                {
                                    fW = Width - (fRealLeftTop) - fz2;
                                }
                                e.Graphics.FillRectangle(sbFore, fz2, fRealLeftTop, fW, fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                                e.Graphics.FillRectangle(sbFore, fRealLeftTop, fRealLeftTop, Convert.ToSingle(fz), fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                                if (!bAcrossed)
                                {
                                    e.Graphics.FillRectangle(sbFore, fRealLeftTop, fRealLeftTop, Convert.ToSingle(fz), fRealHeight);
                                }
                                else
                                {
                                    e.Graphics.FillRectangle(sbFore, Convert.ToSingle(fz) + fRealLeftTop, fRealLeftTop, fRealWidth - Convert.ToSingle(fz), fRealHeight);
                                }
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Blocks)
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                                float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                                float fNowBlocks = Convert.ToSingle(fz) / fBlockSize;

                                for (int i = 0; i < fNowBlocks; i++)
                                {
                                    e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                }
                            }
                            else
                            {
                                double fz = fRange * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                                float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                                float fNowBlocks = Convert.ToSingle(fz) / fBlockSize;
                                if (!bAcrossed)
                                {
                                    for (int i = 0; i < fNowBlocks; i++)
                                    {
                                        e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                    }
                                }
                                else
                                {
                                    for (int i = Convert.ToInt32(fNowBlocks); i < fBlocks; i++)
                                    {
                                        e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (L_MarqueeStyle == MarqueeStyle.Swing)
                            {
                                e.Graphics.FillRectangle(sbFore, Convert.ToSingle(dMarqueeValue) + fRealLeftTop, fRealLeftTop, fSlider, fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Cross)
                            {
                                float fz2 = Convert.ToSingle(dMarqueeValue) + fRealLeftTop;
                                float fW = fSlider;
                                if (fz2 < fRealLeftTop)
                                {
                                    fW = fz2 + fSlider - (fRealLeftTop);
                                    fz2 = fRealLeftTop;
                                }
                                if (fz2 + fSlider > Width - (fRealLeftTop))
                                {
                                    fW = Width - (fRealLeftTop) - fz2;
                                }
                                e.Graphics.FillRectangle(sbFore, fz2, fRealLeftTop, fW, fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                            {
                                e.Graphics.FillRectangle(sbFore, fRealLeftTop, fRealLeftTop, Convert.ToSingle(dMarqueeValue), fRealHeight);
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                            {
                                if (!bAcrossed)
                                {
                                    e.Graphics.FillRectangle(sbFore, fRealLeftTop, fRealLeftTop, Convert.ToSingle(dMarqueeValue), fRealHeight);
                                }
                                else
                                {
                                    e.Graphics.FillRectangle(sbFore, Convert.ToSingle(dMarqueeValue) + fRealLeftTop, fRealLeftTop, fRealWidth - Convert.ToSingle(dMarqueeValue), fRealHeight);
                                }
                            }
                            else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Blocks)
                            {
                                float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                                float fNowBlocks = Convert.ToSingle(dMarqueeValue) / fBlockSize;

                                for (int i = 0; i < fNowBlocks; i++)
                                {
                                    e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                }
                            }
                            else
                            {
                                float fBlocks = Convert.ToSingle(fRealWidth) / fBlockSize;
                                float fNowBlocks = Convert.ToSingle(dMarqueeValue) / fBlockSize;
                                if (!bAcrossed)
                                {
                                    for (int i = 0; i < fNowBlocks; i++)
                                    {
                                        e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                    }
                                }
                                else
                                {
                                    for (int i = Convert.ToInt32(fNowBlocks); i < fBlocks; i++)
                                    {
                                        e.Graphics.FillRectangle(sbFore, i * fBlockSize + fRealLeftTop, fRealLeftTop, fBlockSize * 2 / 3, fRealHeight);
                                    }
                                }
                            }
                        }
                    }
                    #endregion
                    #region Helical
                    else if (_ProgressStyle == ProgressStyle.Helical)
                    {                        
                        //这里为了获得更好看的效果，要多加2个Block
                        float fBmpWidth = fRealWidth + fBlockSize*2;
                        float fBlocks = fBmpWidth / fBlockSize;

                        Bitmap bmp = new Bitmap(Convert.ToInt32(fBmpWidth), Convert.ToInt32(fRealHeight));
                        GraphicsPath gp = new GraphicsPath();
                        for (int i = 0; i < fBlocks; i++)
                        {
                            gp.AddPolygon(new PointF[]
                            {
                            //top-right
                            new PointF(fBmpWidth - i * fBlockSize ,0),
                            //bottom-right
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize,fRealHeight),
                            //bottom-left
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize*5/3,fRealHeight),
                            //top-left
                            new PointF(fBmpWidth - i * fBlockSize - fBlockSize*2/3,0),

                            });
                            gp.CloseFigure();
                        }
                        gp.CloseAllFigures();

                        Graphics g = Graphics.FromImage(bmp);
                        g.SmoothingMode = SmoothingMode.HighQuality;
                        g.FillPath(sbFore, gp);
                        gp.Dispose();
                        g.Dispose();
                        e.Graphics.DrawImage(bmp, new RectangleF(fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight), new RectangleF(Convert.ToSingle(dMarqueeValue) + fBlockSize, 0, fRealWidth, bmp.Height), GraphicsUnit.Pixel);
                        bmp.Dispose();
                    }
                    #endregion
                }
            }
            #endregion
            #region Arc
            else if (_ProgressType == ProgressType.Arc)
            {
                //画内部
                if (bDrawInner)
                {
                    e.Graphics.FillEllipse(sbInner, rectInner);
                }

                //画线
                if (bDrawSurroundLine)
                {
                    e.Graphics.DrawEllipse(penSurrondLine, retcSurrondLine.X, retcSurrondLine.Y, retcSurrondLine.Width, retcSurrondLine.Height);
                }

                #region Continuous
                if (_ProgressStyle == ProgressStyle.Continuous)
                {
                    float fProgress = Convert.ToSingle(_Value * 3.6);
                    if (fProgress > 0)
                    {
                        e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, fProgress);
                    }
                }
                #endregion
                #region Blocks
                else if (_ProgressStyle == ProgressStyle.Blocks)
                {
                    //获取方块数
                    float fBlocks = 360.0f / fBlockSize;
                    int iShowBlocks = Convert.ToInt32(fBlocks * _Value / 100);
                    //画进度
                    for (int i = 0; i < iShowBlocks; i++)
                    {
                        //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                        //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                        e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                    }
                }
                #endregion
                #region BlocksToContinuous
                else if (_ProgressStyle == ProgressStyle.BlocksToContinuous)
                {
                    //获取方块数
                    float fBlocks = 360.0f / fBlockSize;
                    if (_Value <= 50)
                    {
                        int iShowBlocks = Convert.ToInt32(fBlocks * _Value * 2 / 100);
                        //画进度
                        for (int i = 0; i < iShowBlocks; i++)
                        {
                            //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                            //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                        }
                    }
                    else
                    {
                        int iShowBlocks = Convert.ToInt32(fBlocks * (_Value - 50f) * 2 / 100);
                        //画进度
                        for (int i = iShowBlocks; i < Convert.ToInt32(fBlocks); i++)
                        {
                            //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                            //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                        }

                        float fProgress = (_Value - 50f) * 2 * 3.6f;
                        if (fProgress > 0)
                        {
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, fProgress);
                        }
                    }
                }
                #endregion
                #region Marquee
                else if (_ProgressStyle == ProgressStyle.Marquee)
                {
                    if (L_MarqueeType == MarqueeType.SlowInSlowOut)
                    {
                        if (L_MarqueeStyle == MarqueeStyle.Cross || L_MarqueeStyle == MarqueeStyle.Swing)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2 + 270 - fSlider / 2;
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, Convert.ToSingle(fz), fSlider);
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            if (!bAcrossed)
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, Convert.ToSingle(fz));
                            }
                            else
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + Convert.ToSingle(fz), Convert.ToSingle(360 - fz));
                            }
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, Convert.ToSingle(fz));
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            float fBlocks = Convert.ToSingle(fz / fBlockSize);
                            float fAllBlocks = 360.0f / fBlockSize;
                            if (!bAcrossed)
                            {
                                for (int i = 0; i < fBlocks; i++)
                                {
                                    e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                            else
                            {
                                for (float i = fAllBlocks - 1; i >= fBlocks; i--)
                                {
                                    e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                        }
                        else
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            float fBlocks = Convert.ToSingle(fz / fBlockSize);

                            for (int i = 0; i < fBlocks; i++)
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                            }

                        }

                    }
                    else
                    {
                        if (L_MarqueeStyle == MarqueeStyle.Cross || L_MarqueeStyle == MarqueeStyle.Swing)
                        {
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, Convert.ToSingle(dMarqueeValue), fSlider);
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                        {
                            if (!bAcrossed)
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, Convert.ToSingle(dMarqueeValue));
                            }
                            else
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + Convert.ToSingle(dMarqueeValue), Convert.ToSingle(360 - dMarqueeValue));
                            }
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                        {
                            e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270, Convert.ToSingle(dMarqueeValue));
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks)
                        {
                            float fBlocks = Convert.ToSingle(dMarqueeValue / fBlockSize);
                            float fAllBlocks = 360.0f / fBlockSize;
                            if (!bAcrossed)
                            {
                                for (int i = 0; i < fBlocks; i++)
                                {
                                    e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                            else
                            {
                                for (float i = fAllBlocks - 1; i >= fBlocks; i--)
                                {
                                    e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                        }
                        else
                        {
                            float fBlocks = Convert.ToSingle(dMarqueeValue / fBlockSize);
                            for (int i = 0; i < fBlocks; i++)
                            {
                                e.Graphics.DrawArc(penSurroundArc, retcSurrondArc, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                            }
                        }
                    }

                }
                #endregion
            }
            #endregion
            #region Pie
            else if (_ProgressType == ProgressType.Pie)
            {
                //画内部
                if (bDrawInner)
                {
                    e.Graphics.FillEllipse(sbInner, rectInner);
                }

                //画线
                if (bDrawSurroundLine)
                {
                    e.Graphics.DrawEllipse(penSurrondLine, retcSurrondLine.X, retcSurrondLine.Y, retcSurrondLine.Width, retcSurrondLine.Height);
                }

                #region Continuous
                if (_ProgressStyle == ProgressStyle.Continuous)
                {
                    //画进度
                    //360/100;
                    float fProgress = Convert.ToSingle(_Value * 3.6);
                    if (fProgress > 0)
                    {
                        //如果不进行fProgress>0的判断，在设计器调整大小时如果处于其它控件上方时会出来异常，导致绘制失败，虽然仍可以进行编译和使用。
                        e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, fProgress);
                    }
                }
                #endregion
                #region Blocks
                else if (_ProgressStyle == ProgressStyle.Blocks)
                {
                    //获取方块数
                    float fBlocks = 360.0f / fBlockSize;
                    int iShowBlocks = Convert.ToInt32(fBlocks * _Value / 100);
                    //画进度
                    for (int i = 0; i < iShowBlocks; i++)
                    {
                        //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                        //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                        e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                    }
                }
                #endregion
                #region 
                else if (_ProgressStyle == ProgressStyle.BlocksToContinuous)
                {
                    //获取方块数
                    float fBlocks = 360.0f / fBlockSize;
                    if (_Value <= 50)
                    {
                        int iShowBlocks = Convert.ToInt32(fBlocks * _Value * 2 / 100);
                        //画进度
                        for (int i = 0; i < iShowBlocks; i++)
                        {
                            //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                            //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                        }
                    }
                    else
                    {
                        int iShowBlocks = Convert.ToInt32(fBlocks * (_Value - 50f) * 2 / 100);
                        //画进度
                        for (int i = iShowBlocks; i < Convert.ToInt32(fBlocks); i++)
                        {
                            //对于DrawArc而言，会产生在超过360度而导致重叠，所以需要判断，同时使用360-1，以防止相接。
                            //后续可以再进行优化：如果所分配的不正好是3的倍数，则取一相近值，使其能被3整除。
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                        }

                        //画进度
                        //360/100;
                        float fProgress = (_Value - 50f) * 2 * 3.6f;
                        if (fProgress > 0)
                        {
                            //如果不进行fProgress>0的判断，在设计器调整大小时如果处于其它控件上方时会出来异常，导致绘制失败，虽然仍可以进行编译和使用。
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, fProgress);
                        }
                    }
                }
                #endregion
                #region Marquee
                else if (_ProgressStyle == ProgressStyle.Marquee)
                {
                    if (L_MarqueeType == MarqueeType.SlowInSlowOut)
                    {
                        if (L_MarqueeStyle == MarqueeStyle.Cross || L_MarqueeStyle == MarqueeStyle.Swing)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2 + 270 - fSlider / 2;
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, Convert.ToSingle(fz), fSlider);
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            if (!bAcrossed)
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, Convert.ToSingle(fz));
                            }
                            else
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + Convert.ToSingle(fz), Convert.ToSingle(360 - fz));
                            }
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, Convert.ToSingle(fz));
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks)
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            float fBlocks = Convert.ToSingle(fz / fBlockSize);
                            float fAllBlocks = 360.0f / fBlockSize;
                            if (!bAcrossed)
                            {
                                for (int i = 0; i < fBlocks; i++)
                                {
                                    e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                            else
                            {
                                for (float i = fAllBlocks - 1; i >= fBlocks; i--)
                                {
                                    e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                        }
                        else
                        {
                            double fz = 360 * (Math.Sin((dMarqueeValue - 90) * Math.PI / 180) + 1) / 2;
                            float fBlocks = Convert.ToSingle(fz / fBlockSize);

                            for (int i = 0; i < fBlocks; i++)
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                            }

                        }
                    }
                    else
                    {
                        if (L_MarqueeStyle == MarqueeStyle.Cross || L_MarqueeStyle == MarqueeStyle.Swing)
                        {
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, Convert.ToSingle(dMarqueeValue), fSlider);
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                        {
                            if (!bAcrossed)
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, Convert.ToSingle(dMarqueeValue));
                            }
                            else
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + Convert.ToSingle(dMarqueeValue), Convert.ToSingle(360 - dMarqueeValue));
                            }
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                        {
                            e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270, Convert.ToSingle(dMarqueeValue));
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks)
                        {
                            float fBlocks = Convert.ToSingle(dMarqueeValue / fBlockSize);
                            float fAllBlocks = 360.0f / fBlockSize;
                            if (!bAcrossed)
                            {
                                for (int i = 0; i < fBlocks; i++)
                                {
                                    e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                            else
                            {
                                for (float i = fAllBlocks - 1; i >= fBlocks; i--)
                                {
                                    e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                                }
                            }
                        }
                        else
                        {
                            float fBlocks = Convert.ToSingle(dMarqueeValue / fBlockSize);
                            for (int i = 0; i < fBlocks; i++)
                            {
                                e.Graphics.FillPie(sbFore, fRealLeftTop, fRealLeftTop, fRealWidth, fRealHeight, 270 + i * fBlockSize, fBlockSize * 2 / 3 > (359.0f - i * fBlockSize) ? (359.0f - i * fBlockSize) : fBlockSize * 2 / 3);
                            }
                        }
                    }

                }
                #endregion
            }
            #endregion

            #region Text

            if (_ProgressText != ProgressText.None)
            {
                if (_ProgressStyle != ProgressStyle.Marquee)
                {
                    string sText = _ProgressText == ProgressText.Number ? _Value.ToString() : _Value.ToString() + "%";
                    SizeF sizeText = e.Graphics.MeasureString(sText, Font);
                    float fx = (Width - sizeText.Width) / 2f;
                    float fy = (Height - sizeText.Height) / 2f;
                    e.Graphics.DrawString(sText, Font, _TextColor == Color.Empty ? sbFore : sbText, fx, fy);
                }
            }

            #endregion

            sbInner.Dispose();
            sbFore.Dispose();
            sbText.Dispose();
        }

        private void TTimer_Tick(object sender, EventArgs e)
        {
            //进度实际绘制宽度、高度
            float fRealWidth = Width - _DrawMargin * 2;
            float fRealHeight = Height - _DrawMargin * 2;

            float fSlider = Convert.ToSingle(fRealWidth) / 5;
            if (_ProgressType != ProgressType.Bar)
            {
                //360/5
                fSlider = 72;
            }

            float fRange = 0;
            if (L_MarqueeStyle == MarqueeStyle.Swing) fRange = fRealWidth - fSlider;
            else if (L_MarqueeStyle == MarqueeStyle.Cross) fRange = fRealWidth + fSlider;
            else fRange = fRealWidth;

            double dStep = Convert.ToDouble(L_MarqueeTime) / 10;
            double dMarqueePer = 180.0 / dStep;
            if (L_MarqueeType == MarqueeType.Even)
            {
                if (_ProgressType == ProgressType.Bar)
                {
                    dMarqueePer = fRange / dStep;
                }
                else
                {
                    dMarqueePer = 360.0 / dStep;
                }
            }

            double dPeriod = 180;
            if (L_MarqueeStyle == MarqueeStyle.Swing || L_MarqueeStyle == MarqueeStyle.Reciprocation_Blocks || L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous) dPeriod = 360;

            if (_ProgressType == ProgressType.Bar)
            {
                if (_ProgressStyle == ProgressStyle.Marquee)
                {
                    if (L_MarqueeType == MarqueeType.SlowInSlowOut)
                    {
                        if (dMarqueeValue > dPeriod)
                        {
                            bAcrossed = !bAcrossed;
                            dMarqueeValue = 0;
                        }
                        dMarqueeValue += dMarqueePer;
                        Invalidate();
                    }
                    else
                    {
                        if (L_MarqueeStyle == MarqueeStyle.Cross)
                        {
                            bToRight = true;
                            if (dMarqueeValue > fRange) dMarqueeValue = -fSlider;
                        }
                        else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks || L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                        {
                            bToRight = true;
                            if (dMarqueeValue > fRange)
                            {
                                bAcrossed = !bAcrossed;
                                dMarqueeValue = 0;
                            }
                        }
                        else
                        {
                            if (dMarqueeValue > fRange) bToRight = false;
                            if (dMarqueeValue < 0) bToRight = true;
                        }
                        dMarqueeValue = bToRight ? dMarqueeValue + dMarqueePer : dMarqueeValue - dMarqueePer;
                        Invalidate();
                    }
                }
                else
                {
                    if (dMarqueeValue > 0) dMarqueeValue = 0;
                    dMarqueeValue -= 1;
                    if (dMarqueeValue <-_BlockSize) dMarqueeValue = 0;                   
                    Invalidate();
                }
            }
            else if (_ProgressType == ProgressType.Arc || _ProgressType == ProgressType.Pie)
            {
                //为了更符合直观认识，即：转到顶部时速度最变到最慢，到最底部时速度加到最快，所以需要从270度开始。
                if (L_MarqueeType == MarqueeType.SlowInSlowOut)
                {
                    if (dMarqueeValue > dPeriod)
                    {
                        bAcrossed = !bAcrossed;
                        dMarqueeValue = 0;
                    }
                    dMarqueeValue += dMarqueePer;
                    Invalidate();
                }
                else
                {
                    if (L_MarqueeStyle == MarqueeStyle.Cross)
                    {
                        bToRight = true;
                        if (dMarqueeValue > 360) dMarqueeValue = 0;
                    }
                    else if (L_MarqueeStyle == MarqueeStyle.Across_Blocks || L_MarqueeStyle == MarqueeStyle.Across_Continuous)
                    {
                        bToRight = true;
                        if (dMarqueeValue > 360)
                        {
                            bAcrossed = !bAcrossed;
                            dMarqueeValue = 0;
                        }
                    }
                    else if (L_MarqueeStyle == MarqueeStyle.Reciprocation_Blocks || L_MarqueeStyle == MarqueeStyle.Reciprocation_Continuous)
                    {
                        if (dMarqueeValue > 360) bToRight = false;
                        if (dMarqueeValue < 0) bToRight = true;
                    }
                    else
                    {
                        if (dMarqueeValue > 270 - fSlider / 2) bToRight = false;
                        if (dMarqueeValue < -90 - fSlider / 2) bToRight = true;
                    }

                    dMarqueeValue = bToRight ? dMarqueeValue + dMarqueePer : dMarqueeValue - dMarqueePer;
                    Invalidate();
                }
            }

        }

    }
}
